/*
 * @(#)ezLicenseExtAPI.h    1.00   10/27/04
 *
 * Copyright (c) 2004-Present Agilis Software LLC, Santa Clara, CA, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Agilis Software LLC ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only
 * in accordance with the terms of the license agreement you entered
 * into with Agilis Software LLC.
 */

/**
 * This is an extended EasyLicenser API that provides
 * higher-level functionality, reducing the level of development
 * effort involved in integrating an application.  The extended
 * functionality includes:<br>
 * 1. Optional deferred node locking.<br>
 * 2. Optional deferred time-limited licensing (requires
 *    an appropriate time-limited-trial-generator license key
 *    from Agilis).<br>
 * The extended API provides high level mechanisms for:<br>
 * 1. Activation - use during product installation or activation
 *    upon first use.<br>
 * 2. Steady state license check - use wherever license checks
 *    are to be performed during normal license operations.<br>
 * 3. Deactivation - use during product uninstallation or
 *    explicit deactivation.<br><br>
 * 
 * The API's conceptually extends the EzLicenseInfo "class".
 *
 * How to use for activation:
 * <br>
 * Invoke the "activate" API call:
 *		<code>void * api = activateExtAPILicense(licenseKey, auxKey, flags, etc.);</code><br>
 * The returned api handle can then be introspected in the same manner as the underlying EzLicenseInfo,
 * as well as to obtain the created key cookie and warning status code.
 * <br>
 * How to use for checking a license in the steady state:
 * <br>
 * Invoke the "check" API call:
 *		<code>
 *		void *api = checkExtAPILicense(key-cookie, flags, etc.);
 *		</code><br>
 * The returned api handle can then be introspected in the same manner as the underlying EzLicenseInfo,
 * as well as to obtain the warning status code.
 * <br><br>
 * How to use for deactivating a license:
 * <br>
 * Invoke the "deactivate" API call:
 *      <code>int warningBitmap = deactivateExtAPILicense(key-cookie, flags, etc.);</code><br>
 * The returned warning bitmap contains the appropriate EzLicenseInfo warning codes, or 0 if success.
 *
 * How to use for deferred time limited licensing, then later switch to perpetual licensing:<br>
 * 1. Generate a single perpetual key with the EZLM:KEY-EXPIRE-DAYS option described below, set to
 *    the desired relative expiration days.  The key is not node-locked.<br>
 * 2. Use the above key together with the Agilis-supplied enabling key in the activation API call.<br>
 * 3. When re-activating with a perpetual key:<br>
 * (a) the perpetual key must <b><i>not</i></b> have the above option specified.<br>
 * (b) invoke the deactivation API call, then invoke the activation API call, this time with the new
 *     key.  The enabling key need not be specified in this call, and if it is, it is ignored upon
 *     detection of the perpetual key.<br><br>
 *
 */


#ifndef __EZLICENSEEXTAPI_H
#define  __EZLICENSEEXTAPI_H

#include <ezLicenseInfo.h>

#ifdef __cplusplus
extern "C" {
#endif


/* CONSTANTS */


/*
 * Relative-expiration-days product option name.  The option may be specified
 * for a key intended to be used in conjunction with an Agilis-supplied enabling
 * key for the purpose of specifying a shorter expiration period than the limit
 * that is programmed into the enabling key.  If the option is specified for a 
 * conventional per-application-installation key and the key is not accompanied by
 * an enabling key, it is an error.  If neither this option nor the key shelf
 * life option is specified, and the key is accompanied by an enabling key,
 * the latter is ignored and the key is considered a conventional per-
 * application-installation key.
 */
#define EZLM_EXPDAYS_OPTION_NAME "EZLM:KEY-EXPIRE-DAYS"
/*
 * Key-shelf-life-in-hours product option name.  The value represents the time in
 * hours an end user has available to activate a license with the key from the
 * time the key is generated by the vendor.  The option may be specified
 * for a key intended to be used in conjunction with an Agilis-supplied enabling
 * key for the purpose of specifying a deferred-node-locked license.
 * If the option is specified for a conventional per-application-instance key 
 * and the key is not accompanied by an enabling key, it is an error.
 */
#define EZLM_SHELFLIFE_OPTION_NAME "EZLM:KEY-SHELF-LIFE-HRS"

#define EZLICENSE_PRODUCT_NAME "EasyLicenser"

/* LICENSE STATE MANAGEMENT */


/*
* Free license extended API context and its underlying license info context.  
* Note that allocation is via license activation / check factory API calls.
* @param licenseExtAPIHandle: license extended API context handle
*/
DLLEXPORT void CALLING_CONVENTION freeLicenseExtAPIHandle(void *licenseExtAPIHandle);


/* LICENSE STATE ACCESSORS */


/**
* getLicenseExtAPIStatusCode:
* Returns status code from last extended API operation.
* @param licenseExtAPIHandle :	license extended API context handle
* @return: status code: one of EZLIC_EXC_*
*/
DLLEXPORT int CALLING_CONVENTION getLicenseExtAPIStatusCode(void *licenseExtAPIHandle);


/**
* getLicenseExtAPILicenseInfo:
* returns license info handle contained in extended API context for 
* subsequent use in direct calls defined in EzLicenseInfo.h.
* @param licenseExtAPIHandle:	license extended API context handle
* @return: status code: one of EZLIC_EXC_*
*/
DLLEXPORT void *CALLING_CONVENTION getLicenseExtAPILicenseInfo(void *licenseExtAPIHandle);


/**
* getLicenseExtAPILicenseKeyStr:
* returns license key from extended API handle into supplied buffer
*	@param licenseExtAPIHandle: extended license API handle.
*   @param strBuffer: string buffer into which to return license key
*   @param bufferSize: size of string buffer
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getLicenseExtAPILicenseKeyStr(
										void *licenseExtAPIHandle, 
										char strBuffer[],
										size_t bufferSize);


			
/**
* getLicenseExtAPIUserAppstateStr:
* returns user-defined application state embedded in key cookie from extended API 
* handle into supplied buffer, empty string if none.
*	@param licenseExtAPIHandle: extended license API handle.
*   @param strBuffer: string buffer into which to return user application state
*   @param bufferSize: size of string buffer
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getLicenseExtAPIUserAppstateStr(
										void *licenseExtAPIHandle, 
										char strBuffer[],
										size_t bufferSize);

/**
* getLicenseExtAPIUserhostStr:
* returns user/host name embedded in key cookie from extended API 
* handle into supplied buffer, empty string if none.
*	@param licenseExtAPIHandle: extended license API handle.
*   @param strBuffer: string buffer into which to return user and host name.
*   @param bufferSize: size of string buffer
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getLicenseExtAPIUserhostStr(
										void *licenseExtAPIHandle, 
										char strBuffer[],
										size_t bufferSize);


/**
* getLicenseExtAPIActivationTs:
* Returns activation timestamp embedded in key cookie, 0 if not activated.
* @param licenseExtAPIHandle :	license extended API context handle
* @return: activation timestamp embedded in key cookie state.
*/
DLLEXPORT time_t CALLING_CONVENTION getLicenseExtAPIActivationTs(void *licenseExtAPIHandle);


/**
* getLicenseExtAPIExpirationTs:
* Returns expiration timestamp embedded in key cookie, 0 if not relative-time-limited.
* @param licenseExtAPIHandle :	license extended API context handle
* @return: expiration timestamp embedded in key cookie state.
*/
DLLEXPORT time_t CALLING_CONVENTION getLicenseExtAPIExpirationTs(void *licenseExtAPIHandle);


/* License Key management API calls */

/**
 * activateExtAPILicense:
 * Activate the specified license key in the current machine under
 * the current operating system account, and allocate a license context.
 * Validate the specified license key, and initialize the activation
 * state according to the input parameters for the purpose of
 * starting the clock ticking if this is a deferred time-limited
 * license (an auxiliary time-limited-license key is specified as
 * an input parameter) and initializing usage counts for metered
 * licensing.
 * @param licenseKey: vendor-provided license key.
 * @param auxKey: Agilis-provided auxiliary license key needed to
 *          enable certain functions such as deferred-time-limited
 *          licensing and deferred node locking.
 * @param customKeyHandler: optional custom license 
 * 			key handler for processing custom key in
 *			an ISV specific manner.  Null if not
 *			specified.
 * @param customKeyContext: optional run time context
 *			for custom license key handler above.
 * @param userHostNet: user or host or network name, for a 
 *          user / node	locked / floating license.  This is required
 *          to match what is in the license key.  It may be null if
 *          the key was generated with "enforce os check" set to true.
 *          When using deferred node locking, the supplied key does
 *          not have to be node locked, and the userHostNet parameter
 *          does not have to match the user/machine on which the
 *          application is running.
 * @param product: product name.  Must match what's in the key if
 *		    specified.  If not specified, app password can't be
 *          specified either.
 * @param appPassword: application password's public key.  
 *          One must be supplied if the key contains a password.
 * @param userAppstate: stringified generic license related 
 *			application state to embed in the key cookie for 
 *          later retrieval.  If specified, the only way to
 *          later retrieve the state is with this same extended API.
 *          That is, the information is inaccessible from the normal
 *          key cookie API's that are ordinarily used with the
 *          core API's.
 * @param hfileDirectory: preferred hidden-file directory.  If
 *          null, and if hidden-file checking is enabled, the
 *          current working directory is used.
 * @param alwaysCheckTimestamp: whether to unconditionally check
 *          the hidden file's timestamp against the system clock.
 *          If false, the timestamp is never checked, not even for 
 *          time limited licenses.
 * @param deferredNodeLock: whether to perform deferred node locking,
 *          so as to prevent the activated key cookie from being transported
 *          to multiple machines for multiple concurrent uses.
 *          The flag is ignored if an enabling key is not supplied.
 * @param pLicenseExtendedAPIHandle: pointer to a "void *" variable to
 *          receive an allocated extended-API license handle. The handle
 *          contains the key cookie, warning status code etc, if the 
 *          license key check is successful.  The caller is responsible
 *          for freeing the handle (using freeLicenseExtAPIHandle()) 
 *          after license activations and / or checks are completed.
 * @return status code: one of EZLIC_EXC_* defined in ezLicStatusCodes.h
 */
DLLEXPORT int CALLING_CONVENTION activateExtAPILicense(
									   const char*		licenseKey,	
									   const char*		auxKey,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const char*		userHostNet, 
									   const char*		product,
									   const char*		appPassword,
									   const char*		appState,
									   const char*		hfileDirectory,
									   boolean			alwaysCheckTimestamp,
									   boolean 			deferredNodeLock,
									   void** 			pLicenseExtAPIHandle
									   );


/**
 * checkExtAPILicense:
 * Check the specified license key cookie
 * against the provided info for key violations,
 * and set the class context based on the key.
 * This version matches product name and application
 * password, and also accepts an application context
 * for embedding into the key cookie.
 * Throw an appropriate exception if the key is
 * invalid or there is a license violation.
 * Returns warning code bitmap
 * to indicate almost-out-of-quota, expiring, etc.
 * conditions that are not yet errors.
 * @param keyCookie: the key cookie that was returned
 *          from the previous call to this method for 
 *          this key, or from the activateLicense() 
 *          method if none.
 * @param customKeyHandler: optional custom license 
 * 			key handler for processing custom key in
 *			an ISV specific manner.  Null if not
 *			specified.
 * @param customKeyContext: optional run time context
 *			for custom license key handler above.
 * @param daysLeftThreshold: threshold for # of days
 *			remaining in a time-limited license in order
 *			to trigger an "about-to-expire" warning.
 * @param quotaUsageToDate: amount of consumed quota,
 *			if this is expected to be a metered license
 *          and the value is positive.  If the value is
 *          negative, it represents the incremental 
 *          quota consumption by which to reduce the 
 *          available balance that is stored in the key cookie.
 * @param quotaThreshold: threshold for # of license
 *			units remaining in a metered license in order
 *			to trigger an "about-to-expire" warning.
 * @param daysGracePeriod: grace period for time limited license
 *          - if today is past expiration date by
 *          less than this many days, set a warning instead of
 *          throwing an exception.
 * @param quotaGrace: grace quota - if quota is exceeded by
 *          less than this amount, set a warning instead of
 *          throwing an exception.
 * @param userHostNet: user or host or network name, for a 
 *          user / node	locked / floating license.
 * @param currUsageValue: current usage value for
 *			floating / concurrent-user / cpu / mhz
 *			usage models.
 * @param product: product name.  Must match what's in the key if
 *		    specified.  If not specified, app password can't be
 *          specified either.
 * @param appPassword: application password's public key.  
 *          One must be supplied if the key contains a password.
 * @param appState: stringified generic license related 
 *			application state to embed in the key cookie for 
 *          later retrieval.
 * @param hfileDirectory: preferred hidden-file directory.  If
 *          null, and if hidden-file checking is enabled, the
 *          Java "user.home" property is used.
 * @param alwaysCheckTimestamp: whether to unconditionally check
 *          the hidden file's timestamp against the system clock.
 *          If FALSE, the timestamp is never checked, not even for
 *          time limited licenses.
 * @param pLicenseExtendedAPIHandle: pointer to a "void *" variable to
 *          receive an allocated extended-API license handle. The handle
 *          contains the key cookie, warning status code etc, if the 
 *          license key check is successful.  The caller is responsible
 *          for freeing the handle (using freeLicenseExtAPIHandle()) 
 *          after license checks are completed.
 * @return status code: one of EZLIC_EXC_* defined in ezLicStatusCodes.h
 */
DLLEXPORT int CALLING_CONVENTION checkExtAPILicense(
									   const char*				keyCookie,	
									   ezLicCustomKeyHandler_t	customKeyHandler,
									   void*					customKeyContext,
									   int						daysLeftThreshold,
									   long						quotaUsageToDate,
									   long						quotaThreshold,
									   int						daysGracePeriod,
									   long						quotaGrace,
									   const char*				userHostNet,
									   long						currUsageValue,
									   const char*				product,
									   const char*				appPassword,
									   const char*				appState,
									   const char*				hfileDirectory,
									   boolean					alwaysCheckTimestamp,
									   void** 					pLicenseExtAPIHandle
									   );

/**
 * Deactivate license.
 * Typically invoked at uninstallation time or immediately prior to reactivation.
 * @param keyCookie: last-valid key cookie
 * @param productName: product name
 * @param appPassword: application password, same as that specified
 * to activateLicense() and checkLicense().
 * @param hfileDirectory: directory where hidden file is supposed to be
 * @param check: whether to do a timestamp check
 * @return error / warning code bitmap.
 **/
DLLEXPORT int CALLING_CONVENTION deactivateExtAPILicense(
									char 	   *keyCookie, 
									const char *productName, 
									const char *appPassword,
									const char *hfileDirectory, 
									boolean 	check);


/**
* getLicenseExtAPIKeyCookieStr:
* returns keyCookie embedded in key cookie from extended API 
* handle into supplied buffer, empty string if none.
*	@param licenseExtAPIHandle: extended license API handle.
*   @param strBuffer: string buffer into which to return user and host name.
*   @param bufferSize: size of string buffer
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getLicenseExtAPIKeyCookieStr(
										void *licenseExtAPIHandle, 
										char strBuffer[],
										size_t bufferSize);


#ifdef __cplusplus
}
#endif

#endif

	
